'''
        Read Gyro and Accelerometer by Interfacing Raspberry Pi with MPU6050 using Python
    http://www.electronicwings.com
'''

import smbus            #import SMBus module of I2C
from time import sleep
import math
import datetime
import picamera


#some MPU6050 Registers and their Address
PWR_MGMT_1   = 0x6B
SMPLRT_DIV   = 0x19
CONFIG       = 0x1A
GYRO_CONFIG  = 0x1B
INT_ENABLE   = 0x38
ACCEL_XOUT_H = 0x3B
ACCEL_YOUT_H = 0x3D
ACCEL_ZOUT_H = 0x3F
GYRO_XOUT_H  = 0x43
GYRO_YOUT_H  = 0x45
GYRO_ZOUT_H  = 0x47


#-------------------vars-----2--------------------------------


Register_A     = 0              #Address of Configuration register A
Register_B     = 0x01           #Address of configuration register B
Register_mode  = 0x02           #Address of mode register

X_axis_H    = 0x03              #Address of X-axis MSB data register
Z_axis_H    = 0x05              #Address of Z-axis MSB data register
Y_axis_H    = 0x07              #Address of Y-axis MSB data register
declination = -0.00669          #define declination angle of location where measurement going to be done
pi          = 3.14159265359     #define pi value


x_middle=10.0


y_middle=10.0

x=10.0
y=10.0
z=10.0
dx=10.0
dy=10.0
dz=10.0
z_middle=10.0
i=0
porog_delta_magn_xyz=2.0
#----vars2--/2--------------------------------------


#------------------funcs 2-------------------------

def Magnetometer_Init():
        #write to Configuration Register A
        bus.write_byte_data(Device_Address, Register_A, 0x70)

        #Write to Configuration Register B for gain
        bus.write_byte_data(Device_Address, Register_B, 0xa0)

        #Write to mode Register for selecting mode
        bus.write_byte_data(Device_Address, Register_mode, 0)
    
    

def read_raw_data_2(addr):
    
        #Read raw 16-bit value
        high = bus.read_byte_data(Device_Address, addr)
        low = bus.read_byte_data(Device_Address, addr+1)

        #concatenate higher and lower value
        value = ((high << 8) | low)

        #to get signed value from module
        if(value > 32768):
            value = value - 65536
        return value
#-------------------------/funcs 2---------------------


def MPU_Init():
    #write to sample rate register
    bus.write_byte_data(Device_Address, SMPLRT_DIV, 7)
    
    #Write to power management register
    bus.write_byte_data(Device_Address, PWR_MGMT_1, 1)
    
    #Write to Configuration register
    bus.write_byte_data(Device_Address, CONFIG, 0)
    
    #Write to Gyro configuration register
    bus.write_byte_data(Device_Address, GYRO_CONFIG, 24)
    
    #Write to interrupt enable register
    bus.write_byte_data(Device_Address, INT_ENABLE, 1)

def read_raw_data(addr):
    #Accelero and Gyro value are 16-bit
        high = bus.read_byte_data(Device_Address, addr)
        low = bus.read_byte_data(Device_Address, addr+1)
    
        #concatenate higher and lower value
        value = ((high << 8) | low)
        
        #to get signed value from mpu6050
        if(value > 32768):
                value = value - 65536
        return value

#-----/init 2----------------
today = datetime.datetime.today()
#%Y-%m-%d %H:%M:%S
print( today.strftime("%Y-%m-%d %H:%M:%S") )



filename = today.strftime("%Y-%m-%d %H:%M:%S")
outfile = open(filename, 'w')   # открытие файла для записи
#------------------outfile.close()

print (" Reading Data of Magnitometr, Gyroscope and Accelerometer")
print (" порог магн отклонения ?(1-100)")
print("----")

while True:
   try: 
        bus = smbus.SMBus(1)    # or bus = smbus.SMBus(0) for older version boards
        Device_Address = 0x68   # MPU6050 device address

        MPU_Init()    
        #Read Accelerometer raw value
        acc_x = read_raw_data(ACCEL_XOUT_H)
        acc_y = read_raw_data(ACCEL_YOUT_H)
        acc_z = read_raw_data(ACCEL_ZOUT_H)
    
        #Read Gyroscope raw value
        gyro_x = read_raw_data(GYRO_XOUT_H)
        gyro_y = read_raw_data(GYRO_YOUT_H)
        gyro_z = read_raw_data(GYRO_ZOUT_H)
    
        #Full scale range +/- 250 degree/C as per sensitivity scale factor
        Ax = acc_x/16384.0
        Ay = acc_y/16384.0
        Az = acc_z/16384.0
    
        Gx = gyro_x/131.0
        Gy = gyro_y/131.0
        Gz = gyro_z/131.0
    
    
#-------2----------
        bus = smbus.SMBus(1)    # or bus = smbus.SMBus(0) for older version boards
        Device_Address = 0x1e   # HMC5883L magnetometer device address

        Magnetometer_Init()     
        count = 1
        x_middle=0.0
        y_middle=0.0
        z_middle=0.0
        while (count < 11):
        #Read Accelerometer raw value
            sleep(0.07)
                
            x = float(read_raw_data_2(X_axis_H))
            z = float(read_raw_data_2(Z_axis_H))
            y = float(read_raw_data_2(Y_axis_H))
            count = count + 1
            x_middle=x_middle + x
            y_middle=y_middle + y
            z_middle=z_middle + z

                
        x_middle =x_middle/10
        y_middle =y_middle/10
        z_middle =z_middle/10
        
        dx=x-x_middle
        dy=y-y_middle
        dz=z-z_middle

        proc_x=(dx/x_middle)*100.0
        proc_y=(dy/y_middle)*100.0
        proc_z=(dz/z_middle)*100.0    
    
        
        
# -----/2--------- 
        if abs(proc_x) >1.8  or abs(proc_y) > 7 or abs(proc_z) > 1.8:
               
                today = datetime.datetime.today()
                #%Y-%m-%d %H:%M:%S
                print( today.strftime("%Y-%m-%d %H:%M:%S") )
                print("%.2f  " % proc_x,"%.2f  " % proc_y,"%.2f" % proc_z)
                outfile = open(filename, 'a')
                outfile.write(today.strftime("%Y-%m-%d %H:%M:%S"))
                outfile.write('\n')
                outfile.write('%s %5.2f %5.2f %5.2f %s %5.2f %5.2f %5.2f %s %5.2f %5.2f %5.2f\n' %('dMagnit(x,y,z)%', proc_x, proc_y, proc_z,'----Omega(x,y,z)'+ u'\u00b0'+'/s=',Gx, Gy, Gz,'|g(x,y,z)', Ax, Ay,Az))  
                
                with picamera.PiCamera() as camera:
                        camera.start_preview()
                        camera.resolution = (1280, 720)
                        camera.start_recording('1.h264')
                        
                        camera.split_recording('%d.h264' % i)
                        camera.wait_recording(5)
                        camera.stop_recording()
                        i=i+1  
        

        
        sleep(0.1)
    
    

        print("%.2f  " % proc_x,"%.2f  " % proc_y,"%.2f" % proc_z) #--2
        print ( "Gx=%.2f" %Gx, u'\u00b0'+ "/s", "\tGy=%.2f" %Gy, u'\u00b0'+ "/s", "\tGz=%.2f" %Gz, u'\u00b0'+ "/s", "\tAx=%.2f g" %Ax, "\tAy=%.2f g" %Ay, "\tAz=%.2f g" %Az)     
        print("-------------------------------") #--2
        
        #outfile.write('%s %5.2f %5.2f %5.2f %s %5.2f %5.2f %5.2f %s %5.2f %5.2f %5.2f\n' %('dMagnit(x,y,z)%', proc_x, proc_y, proc_z,'----Omega(x,y,z)'+ u'\u00b0'+'/s=',Gx, Gy, Gz,'|g(x,y,z)', Ax, Ay,Az))  
        sleep(0.01)
    
   except KeyboardInterrupt:
       outfile.close()

       break

